/*
    Парус 8 - Панели мониторинга - УДП - Доски задач
    Компонент: Фильтр отбора
*/

//---------------------
//Подключение библиотек
//---------------------

import React, { useState } from "react"; //Классы React
import PropTypes from "prop-types"; //Контроль свойств компонента
import { Chip, Stack, Icon, IconButton, Box, Menu, MenuItem, Typography } from "@mui/material"; //Интерфейсные компоненты
import { APP_STYLES } from "../../../app.styles"; //Типовые стили
import { FilterDialog } from "./components/filter_dialog.js"; //Диалог фильтра

//---------
//Константы
//---------

//Стили
const STYLES = {
    ICON_ORDERS: orders => {
        return orders.length > 0 ? { color: "#1976d2" } : {};
    },
    ORDER_MENU: {
        width: "260px"
    },
    ORDER_MENU_ITEM: {
        display: "flex",
        justifyContent: "space-between"
    },
    FILTERS_STACK: {
        paddingBottom: "5px",
        overflowX: "auto",
        ...APP_STYLES.SCROLL
    },
    FILTER_MAXW: { maxWidth: "99vw" }
};

//--------------------------
//Вспомогательные компоненты
//--------------------------

//Элемент меню сортировок
const SortMenuItem = ({ item, caption, orders, onOrderChanged }) => {
    //Кнопка сортировки
    const order = orders.find(o => o.name == item);

    return (
        <MenuItem sx={STYLES.ORDER_MENU_ITEM} key={item} onClick={() => onOrderChanged(item)}>
            <Typography>{caption}</Typography>
            {order ? order.direction === "ASC" ? <Icon>arrow_upward</Icon> : <Icon>arrow_downward</Icon> : null}
        </MenuItem>
    );
};

//Контроль свойств компонента - Элемент меню сортировок
SortMenuItem.propTypes = {
    item: PropTypes.string.isRequired,
    caption: PropTypes.string.isRequired,
    orders: PropTypes.array,
    onOrderChanged: PropTypes.func.isRequired
};

//Меню сортировок
const SortMenu = ({ menuOrders, onOrdersMenuClose, orders, onOrderChanged }) => {
    return (
        <Menu
            id={`sort_menu`}
            anchorEl={menuOrders.anchorMenuOrders}
            open={menuOrders.openOrders}
            onClose={onOrdersMenuClose}
            MenuListProps={{ sx: STYLES.ORDER_MENU }}
        >
            <SortMenuItem item={"DCHANGE_DATE"} caption={"Дата последнего изменения"} orders={orders} onOrderChanged={onOrderChanged} />
            <SortMenuItem item={"DPLAN_DATE"} caption={"Дата начала работ"} orders={orders} onOrderChanged={onOrderChanged} />
            <SortMenuItem item={"SPREF_NUMB"} caption={"Номер"} orders={orders} onOrderChanged={onOrderChanged} />
        </Menu>
    );
};

//Контроль свойств компонента - Меню сортировок
SortMenu.propTypes = {
    menuOrders: PropTypes.object.isRequired,
    onOrdersMenuClose: PropTypes.func.isRequired,
    orders: PropTypes.array,
    onOrderChanged: PropTypes.func.isRequired
};

//Элемент фильтра
const FilterItem = ({ caption, value, onClick }) => {
    //При нажатии на элемент
    const handleClick = () => (onClick ? onClick() : null);

    //Генерация содержимого
    return (
        <Chip
            label={
                <Stack direction={"row"} alignItems={"center"}>
                    <strong>{caption}</strong>
                    {value ? `:\u00A0${value}` : null}
                </Stack>
            }
            variant="outlined"
            onClick={handleClick}
        />
    );
};

//Контроль свойств компонента - Элемент фильтра
FilterItem.propTypes = {
    caption: PropTypes.string.isRequired,
    value: PropTypes.any,
    onClick: PropTypes.func
};

//---------------
//Тело компонента
//---------------

//Фильтр отбора
const Filter = ({
    isFilterDialogOpen,
    filter,
    docs,
    selectedDoc,
    onFilterChange,
    getDocLinks,
    onFilterOpen,
    onFilterClose,
    onReload,
    orders,
    onOrderChanged,
    ...other
}) => {
    //Состояние меню сортировки
    const [menuOrders, setMenuOrders] = useState({ anchorMenuOrders: null, openOrders: false });

    //По нажатию на открытие меню сортировки
    const handleOrdersMenuButtonClick = event => {
        setMenuOrders(pv => ({ ...pv, anchorMenuOrders: event.currentTarget, openOrders: true }));
    };

    //При закрытии меню
    const handleOrdersMenuClose = () => {
        setMenuOrders(pv => ({ ...pv, anchorMenuOrders: null, openOrders: false }));
    };

    //Генерация содержимого
    return (
        <div>
            {isFilterDialogOpen ? (
                <FilterDialog initial={filter} docs={docs} onFilterChange={onFilterChange} onFilterOpen={onFilterClose} getDocLinks={getDocLinks} />
            ) : null}
            <Box {...other}>
                <Stack direction="row" spacing={1} p={1} alignItems={"center"} sx={STYLES.FILTER_MAXW}>
                    <IconButton title="Обновить" onClick={onReload}>
                        <Icon>refresh</Icon>
                    </IconButton>
                    <IconButton title="Сортировать" sx={STYLES.ICON_ORDERS(orders)} onClick={handleOrdersMenuButtonClick}>
                        <Icon>sort</Icon>
                    </IconButton>
                    <IconButton title="Фильтр" onClick={onFilterOpen}>
                        <Icon>filter_alt</Icon>
                    </IconButton>
                    <Stack direction="row" spacing={1} alignItems={"center"} sx={STYLES.FILTERS_STACK}>
                        {filter.evState ? <FilterItem caption={"Состояние"} value={filter.evState} onClick={onFilterOpen} /> : null}
                        {filter.type ? <FilterItem caption={"Тип"} value={filter.type} onClick={onFilterOpen} /> : null}
                        {filter.catalog ? <FilterItem caption={"Каталог"} value={filter.catalog} onClick={onFilterOpen} /> : null}
                        {filter.wSubcatalogs ? <FilterItem caption={"Включая подкаталоги"} onClick={onFilterOpen} /> : null}
                        {filter.sendPerson ? <FilterItem caption={"Исполнитель"} value={filter.sendPerson} onClick={onFilterOpen} /> : null}
                        {filter.sendDivision ? <FilterItem caption={"Подразделение"} value={filter.sendDivision} onClick={onFilterOpen} /> : null}
                        {filter.sendUsrGrp ? <FilterItem caption={"Группа пользователей"} value={filter.sendUsrGrp} onClick={onFilterOpen} /> : null}
                        {filter.docLink && selectedDoc ? (
                            <FilterItem caption={"Учётный документ"} value={selectedDoc.descr} onClick={onFilterOpen} />
                        ) : null}
                    </Stack>
                </Stack>
                <SortMenu menuOrders={menuOrders} onOrdersMenuClose={handleOrdersMenuClose} orders={orders} onOrderChanged={onOrderChanged} />
            </Box>
        </div>
    );
};

//Контроль свойств компонента - Фильтр отбора
Filter.propTypes = {
    isFilterDialogOpen: PropTypes.bool.isRequired,
    filter: PropTypes.object.isRequired,
    docs: PropTypes.arrayOf(PropTypes.object),
    selectedDoc: PropTypes.object,
    onFilterChange: PropTypes.func.isRequired,
    getDocLinks: PropTypes.func,
    onFilterOpen: PropTypes.func.isRequired,
    onFilterClose: PropTypes.func.isRequired,
    onReload: PropTypes.func.isRequired,
    orders: PropTypes.array,
    onOrderChanged: PropTypes.func.isRequired
};

//--------------------
//Интерфейс компонента
//--------------------

export { Filter };
